package com.scalapay.constraints;

import com.scalapay.model.ScalaPayConfigurationModel;
import de.hybris.platform.core.model.c2l.CurrencyModel;
import de.hybris.platform.servicelayer.i18n.L10NService;
import de.hybris.platform.servicelayer.interceptor.InterceptorContext;
import de.hybris.platform.servicelayer.interceptor.InterceptorException;
import de.hybris.platform.servicelayer.interceptor.ValidateInterceptor;
import de.hybris.platform.store.BaseStoreModel;
import de.hybris.platform.store.services.BaseStoreService;
import org.springframework.beans.factory.annotation.Required;

import java.util.Optional;
import java.util.Set;

public class ScalaConfigurationValidateInterceptor implements ValidateInterceptor<ScalaPayConfigurationModel> {

    private L10NService l10nService;
    private BaseStoreService baseStoreService;

    @Override
    public void onValidate(ScalaPayConfigurationModel model, InterceptorContext interceptorContext) throws InterceptorException {

        Integer minAmount = model.getMinAmount();
        Integer maxAmount = model.getMaxAmount();
        if (minAmount >= maxAmount) {
            throw new InterceptorException(l10nService.getLocalizedString("error.scalapay.configuration.min_max_amount"));
        }

        Set<CurrencyModel> scalaCurrencies = model.getCurrencies();
        if (scalaCurrencies == null || scalaCurrencies.isEmpty()) {
            throw new InterceptorException(l10nService.getLocalizedString("error.scalapay.currencies.empty"));
        }

        Optional<BaseStoreModel> mappedStore = baseStoreService.getAllBaseStores().stream().filter(baseStoreModel ->
                baseStoreModel.getScalapayConfiguration().getCode().equals(model.getCode())).findFirst();
        if (mappedStore.isPresent()) {
            BaseStoreModel baseStoreModel = mappedStore.get();
            Set<CurrencyModel> storeCurrencies = baseStoreModel.getCurrencies();
            for (CurrencyModel scalaCurrency : scalaCurrencies) {
                if (!storeCurrencies.contains(scalaCurrency)) {
                    throw new InterceptorException(l10nService.getLocalizedString(
                            "error.scalapay.configuration.currency.not.in.store",
                            new Object[]{scalaCurrency.getIsocode(), baseStoreModel.getUid()}));
                }
            }
        }
    }

    @Required
    public void setL10nService(final L10NService l10nService) {
        this.l10nService = l10nService;
    }

    @Required
    public void setBaseStoreService(BaseStoreService baseStoreService) {
        this.baseStoreService = baseStoreService;
    }
}
