package com.scalapay.converters;

import com.scalapay.data.ScalaPayStylingData;
import com.scalapay.model.ScalaPayConfigurationModel;
import de.hybris.platform.converters.Populator;
import de.hybris.platform.servicelayer.dto.converter.ConversionException;
import org.apache.commons.lang3.BooleanUtils;
import org.apache.logging.log4j.util.Strings;

import java.util.Map;

import static org.apache.commons.lang.StringUtils.*;

public class ConfigurationToStylingDataPopulator implements Populator<ScalaPayConfigurationModel, Map<String, ScalaPayStylingData>> {

    private static final String PIXELS = "px";
    private static final String PERCENTAGE = "%";
    private static final String HASH = "#";
    private static final String DEFAULT_SIZE = "100px";
    private static final String DEFAULT_LOGO_SIZE = "100&";
    private static final String DEFAULT_PRICE_COLOR = "#145ad1";
    private static final String DEFAULT_LOGO_COLOR = "#000000";
    private static final String PRICE_BOX_SELECTOR_DEFAULT = ".price";
    private static final String NUMBER_OF_PAYMENTS_DEFAULT = "3";
    private static final String LOCALE_DEFAULT = "en";
    private static final String CURRENCY_SEPARATOR_DEFAULT = ",";

    public static final String PAGE_TYPE_PDP = "PDP";
    public static final String PAGE_TYPE_CART = "CART";


    @Override
    public void populate(ScalaPayConfigurationModel model, Map<String, ScalaPayStylingData> stringScalaPayStylingDataMap) throws ConversionException {
        ScalaPayStylingData scalaPayStylingData_PDP = new ScalaPayStylingData();
        scalaPayStylingData_PDP.setEnabled(BooleanUtils.toBoolean(model.getStyling_pdp_showWidget()));
        String styling_sizePx = defaultIfBlank(model.getStyling_pdp_sizePx(), DEFAULT_SIZE);
        scalaPayStylingData_PDP.setSizePx(addSuffixIfMissing(styling_sizePx, PIXELS));
        scalaPayStylingData_PDP.setAmountSelectorArray(model.getStyling_pdp_amountSelectorArray());
        String logoSizePercentage = defaultIfBlank(model.getStyling_pdp_logoSizePercentage(), DEFAULT_LOGO_SIZE);
        scalaPayStylingData_PDP.setLogoSizePercentage(addSuffixIfMissing(logoSizePercentage, PERCENTAGE));
        String priceColorHex = defaultIfBlank(model.getStyling_pdp_priceColorHex(), DEFAULT_PRICE_COLOR);
        scalaPayStylingData_PDP.setPriceColorHex(addPrefixIfMissing(priceColorHex, HASH));
        String styling_logoColorHex = defaultIfBlank(model.getStyling_pdp_logoColorHex(), DEFAULT_LOGO_COLOR);
        scalaPayStylingData_PDP.setLogoColorHex(addPrefixIfMissing(styling_logoColorHex, HASH));
        scalaPayStylingData_PDP.setPriceBoxSelector(defaultIfBlank(model.getStyling_pdp_priceBoxSelector(), PRICE_BOX_SELECTOR_DEFAULT));
        scalaPayStylingData_PDP.setLogoAlignment(defaultIfBlank(model.getStyling_pdp_logoAlignment(), Strings.EMPTY));
        scalaPayStylingData_PDP.setNumberOfPayments(defaultIfBlank(model.getStyling_pdp_numberOfPayments(), NUMBER_OF_PAYMENTS_DEFAULT));
        scalaPayStylingData_PDP.setHideLogo(BooleanUtils.toBoolean(model.getStyling_pdp_hideLogo()));
        scalaPayStylingData_PDP.setHidePrice(BooleanUtils.toBoolean(model.getStyling_pdp_hidePrice()));
        scalaPayStylingData_PDP.setLocale(defaultIfBlank(model.getStyling_pdp_locale(), LOCALE_DEFAULT));
        scalaPayStylingData_PDP.setCurrencySeparator(defaultIfBlank(model.getStyling_pdp_currencySeparator(), CURRENCY_SEPARATOR_DEFAULT));

        stringScalaPayStylingDataMap.put(PAGE_TYPE_PDP, scalaPayStylingData_PDP);

        ScalaPayStylingData scalaPayStylingData_CART = new ScalaPayStylingData();
        scalaPayStylingData_CART.setEnabled(BooleanUtils.toBoolean(model.getStyling_cart_showWidget()));
        styling_sizePx = defaultIfBlank(model.getStyling_cart_sizePx(), DEFAULT_SIZE);
        scalaPayStylingData_CART.setSizePx(addSuffixIfMissing(styling_sizePx, PIXELS));
        scalaPayStylingData_CART.setAmountSelectorArray(model.getStyling_cart_amountSelectorArray());
        logoSizePercentage = defaultIfBlank(model.getStyling_cart_logoSizePercentage(), DEFAULT_LOGO_SIZE);
        scalaPayStylingData_CART.setLogoSizePercentage(addSuffixIfMissing(logoSizePercentage, PERCENTAGE));
        priceColorHex = defaultIfBlank(model.getStyling_cart_priceColorHex(), DEFAULT_PRICE_COLOR);
        scalaPayStylingData_CART.setPriceColorHex(addPrefixIfMissing(priceColorHex, HASH));
        styling_logoColorHex = defaultIfBlank(model.getStyling_cart_logoColorHex(), DEFAULT_LOGO_COLOR);
        scalaPayStylingData_CART.setLogoColorHex(addPrefixIfMissing(styling_logoColorHex, HASH));
        scalaPayStylingData_CART.setPriceBoxSelector(defaultIfBlank(model.getStyling_cart_priceBoxSelector(), PRICE_BOX_SELECTOR_DEFAULT));
        scalaPayStylingData_CART.setLogoAlignment(defaultIfBlank(model.getStyling_cart_logoAlignment(), Strings.EMPTY));
        scalaPayStylingData_CART.setNumberOfPayments(defaultIfBlank(model.getStyling_cart_numberOfPayments(), NUMBER_OF_PAYMENTS_DEFAULT));
        scalaPayStylingData_CART.setHideLogo(BooleanUtils.toBoolean(model.getStyling_cart_hideLogo()));
        scalaPayStylingData_CART.setHidePrice(BooleanUtils.toBoolean(model.getStyling_cart_hidePrice()));
        scalaPayStylingData_CART.setLocale(defaultIfBlank(model.getStyling_cart_locale(), LOCALE_DEFAULT));
        scalaPayStylingData_CART.setCurrencySeparator(defaultIfBlank(model.getStyling_cart_currencySeparator(), CURRENCY_SEPARATOR_DEFAULT));

        stringScalaPayStylingDataMap.put(PAGE_TYPE_CART, scalaPayStylingData_CART);
    }

    private String addSuffixIfMissing(String value, String suffix) {
        if (endsWith(value, suffix)) {
            return value;
        }
        return value + suffix;
    }

    private String addPrefixIfMissing(String value, String prefix) {
        if (startsWith(value, prefix)) {
            return value;
        }
        return prefix + value;
    }
}
