package com.scalapay.facade.impl;

import de.hybris.platform.acceleratorfacades.order.impl.DefaultAcceleratorCheckoutFacade;
import de.hybris.platform.commercefacades.order.data.CCPaymentInfoData;
import de.hybris.platform.commerceservices.service.data.CommerceCheckoutParameter;
import de.hybris.platform.core.model.order.CartModel;
import de.hybris.platform.core.model.order.payment.PaymentInfoModel;
import de.hybris.platform.core.model.order.payment.ScalaPayPaymentInfoModel;
import de.hybris.platform.payment.dto.TransactionStatus;
import de.hybris.platform.payment.model.PaymentTransactionEntryModel;

public class ScalaPayCheckoutFacade extends DefaultAcceleratorCheckoutFacade {

    @Override
    protected CCPaymentInfoData getPaymentDetails() {
        CCPaymentInfoData paymentDetails = super.getPaymentDetails();
        if (paymentDetails == null) {
            final CartModel cart = getCart();
            if (cart != null) {
                final PaymentInfoModel paymentInfo = cart.getPaymentInfo();
                if (paymentInfo instanceof ScalaPayPaymentInfoModel) {
                    paymentDetails = new CCPaymentInfoData();
                    paymentDetails.setCardType("Scala"); // todo will this break something?
                    paymentDetails.setCardNumber(((ScalaPayPaymentInfoModel) paymentInfo).getOrderToken());
                }
            }
        }
        return paymentDetails;
    }

    @Override
    public boolean authorizePayment(String securityCode) {
        final CartModel cartModel = getCart();
        final ScalaPayPaymentInfoModel creditCardPaymentInfoModel = cartModel == null ? null
                : (ScalaPayPaymentInfoModel) cartModel.getPaymentInfo();
        if (checkIfCurrentUserIsTheCartUser() && creditCardPaymentInfoModel != null) {
            final CommerceCheckoutParameter parameter = createCommerceCheckoutParameter(cartModel, true);
            parameter.setSecurityCode(securityCode);
            parameter.setPaymentProvider(getPaymentProvider());
            final PaymentTransactionEntryModel paymentTransactionEntryModel = getCommerceCheckoutService().authorizePayment(
                    parameter);

            return paymentTransactionEntryModel != null
                    && (TransactionStatus.ACCEPTED.name().equals(paymentTransactionEntryModel.getTransactionStatus()) || TransactionStatus.REVIEW
                    .name().equals(paymentTransactionEntryModel.getTransactionStatus()));
        }
        return false;
    }
}
