/*
 * [y] hybris Platform
 *
 * Copyright (c) 2018 SAP SE or an SAP affiliate company. All rights reserved.
 *
 * This software is the confidential and proprietary information of SAP
 * ("Confidential Information"). You shall not disclose such Confidential
 * Information and shall use it only in accordance with the terms of the
 * license agreement you entered into with SAP.
 */
package com.scalapay.service;


import com.scalapay.data.OrderStatus;
import com.scalapay.data.OrderStatusResponse;
import com.scalapay.data.OrderTokenRequest;
import com.scalapay.data.OrderTokenResponse;
import com.scalapay.model.ScalaPayConfigurationModel;
import de.hybris.platform.core.model.order.AbstractOrderModel;
import de.hybris.platform.core.model.order.OrderModel;
import de.hybris.platform.store.BaseStoreModel;

import javax.annotation.Nullable;

public interface ScalapayService {

	String getHybrisLogoUrl(String logoCode);

	void createLogo(String logoCode);

	/**
	 * Get the scalapay configuration for the session cart.
	 * If no session cart exists, it returns the configuration for the first store.
	 * When querying for a specific store or order, use the convenience methods below.
	 */
	ScalaPayConfigurationModel getScalaPayConfiguration();

	ScalaPayConfigurationModel getScalaPayConfiguration(AbstractOrderModel abstractOrder);

	ScalaPayConfigurationModel getScalaPayConfiguration(BaseStoreModel baseStore);

	boolean isEnabled(BaseStoreModel baseStore);

	/**
	 * Generate a Scalapay order token
	 * @param request The order token request
	 * @param store The store
	 * @return The order token response
	 */
	OrderTokenResponse getOrderToken(OrderTokenRequest request, BaseStoreModel store);

	/**
	 * Retrieves the status of an order in Scalapay.
	 * @param orderCode The Hybris order code.
	 * @return The order status response.
	 */
	OrderStatus getOrderStatus(String orderCode);

	/**
	 * Captures the payment for a token
	 * @param token The Scalapay token
	 * @param reference Reference for the capture
	 * @return
	 */
	@Nullable OrderStatusResponse capture(String token, @Nullable String reference);

	/**
	 * Send hybris order code to Scalapay
	 * @param order The order
	 */
	void updateMerchantReference(OrderModel order);

	String refund(String orderCode, double amount, String currencyCode);

	/**
	 * Find a Hybris order/cart for a Scala token.
	 *
	 * @param token The Scalapy token
	 * @return The order, if it exists*
	 */
	AbstractOrderModel getOrderByScalaToken(String token);

}
