package com.scalapay.tags;

import com.scalapay.converters.ConfigurationToStylingDataPopulator;
import com.scalapay.data.ScalaPayStylingData;
import com.scalapay.facade.ScalapayFacade;
import org.apache.commons.lang3.StringUtils;
import org.springframework.web.context.request.RequestContextHolder;
import org.springframework.web.context.request.ServletRequestAttributes;
import org.springframework.web.servlet.support.RequestContextUtils;

import javax.servlet.http.HttpServletRequest;
import java.util.Map;

public class Functions {

    public static boolean isSupportedForSessionCart() {
        ScalapayFacade scalapayFacade = getScalaFacade();
        return scalapayFacade.isEnabled() && scalapayFacade.validForSessionCart();
    }

    public static boolean validForSessionCurrencyAndAmount(double amount) {
        ScalapayFacade scalapayFacade = getScalaFacade();
        return scalapayFacade.isEnabled() &&  scalapayFacade.validForSessionCurrencyAndAmount(amount);
    }

    private static ScalapayFacade getScalaFacade() {
        return getScalaFacade(getCurrentRequest());
    }


    public static String minimumSupportedAmount() {
        ScalapayFacade scalapayFacade = getScalaFacade();
        return String.valueOf(scalapayFacade.getMinimumSupportedAmount());
    }


    public static String maximumSupportedAmount() {
        ScalapayFacade scalapayFacade = getScalaFacade();
        return String.valueOf(scalapayFacade.getMaximumSupportedAmount());
    }

    public static ScalaPayStylingData getScalaPayStylingData(String pageType) {
        if (!StringUtils.equalsAny(pageType, ConfigurationToStylingDataPopulator.PAGE_TYPE_PDP, ConfigurationToStylingDataPopulator.PAGE_TYPE_CART)) {
            // log error
            pageType = ConfigurationToStylingDataPopulator.PAGE_TYPE_PDP;
        }
        ScalapayFacade scalapayFacade = getScalaFacade();
        Map<String, ScalaPayStylingData> stylingData = scalapayFacade.getStylingData();
        return stylingData.getOrDefault(pageType, stylingData.values().stream().findFirst().get());
    }

    protected static HttpServletRequest getCurrentRequest()
    {
        return ((ServletRequestAttributes) RequestContextHolder.currentRequestAttributes()).getRequest();
    }

    protected static ScalapayFacade getScalaFacade(final HttpServletRequest httpRequest) {
        return getSpringBean(httpRequest, "scalapayFacade", ScalapayFacade.class);
    }

    public static <T> T getSpringBean(final HttpServletRequest httpRequest, final String beanName, final Class<T> beanClass)
    {
        return RequestContextUtils.getWebApplicationContext(httpRequest, httpRequest.getSession().getServletContext())
                .getBean(beanName, beanClass);
    }
}
