package com.scalapay.converters;

import com.scalapay.data.OrderTokenRequest;
import com.scalapay.model.ScalaPayConfigurationModel;
import de.hybris.bootstrap.annotations.UnitTest;
import de.hybris.platform.basecommerce.model.site.BaseSiteModel;
import de.hybris.platform.core.model.c2l.CountryModel;
import de.hybris.platform.core.model.c2l.CurrencyModel;
import de.hybris.platform.core.model.order.AbstractOrderEntryModel;
import de.hybris.platform.core.model.order.CartModel;
import de.hybris.platform.core.model.order.OrderEntryModel;
import de.hybris.platform.core.model.order.price.DiscountModel;
import de.hybris.platform.core.model.product.ProductModel;
import de.hybris.platform.core.model.user.AddressModel;
import de.hybris.platform.servicelayer.StubLocaleProvider;
import de.hybris.platform.servicelayer.config.ConfigurationService;
import de.hybris.platform.servicelayer.model.ItemContextBuilder;
import de.hybris.platform.servicelayer.model.ItemModelContext;
import de.hybris.platform.site.BaseSiteService;
import de.hybris.platform.store.BaseStoreModel;
import de.hybris.platform.store.services.BaseStoreService;
import org.apache.commons.io.FileUtils;
import org.codehaus.jackson.map.ObjectMapper;
import org.junit.Test;
import org.junit.runner.RunWith;
import org.mockito.Mock;
import org.mockito.runners.MockitoJUnitRunner;

import java.io.File;
import java.io.IOException;
import java.util.ArrayList;
import java.util.List;
import java.util.Locale;

import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertNotNull;
import static org.mockito.BDDMockito.given;
import static org.mockito.Matchers.any;

@UnitTest
@RunWith(MockitoJUnitRunner.class)
public class CartToTokenRequestPopulatorTest {

    private static final String FIRST_NAME = "test-first";
    private static final String LAST_NAME = "test-last";

    private final CartToTokenRequestPopulator populator = new CartToTokenRequestPopulator();

    @Mock
    private ConfigurationService configurationService;

    @Mock
    private BaseSiteService baseSiteService;

    @Mock
    private BaseStoreService baseStoreService;

    @Mock
    private BaseStoreModel baseStore;

    @Mock
    private ScalaPayConfigurationModel scalaPayConfigurationModel;

    @Mock
    private org.apache.commons.configuration.Configuration configuration;

    @Test
    public void testConvert() throws IOException {

        given(configurationService.getConfiguration()).willReturn(configuration);
        given(configuration.getString(any())).willReturn("testConfigValue");
        BaseSiteModel baseSiteModel = new BaseSiteModel();
        given(baseSiteService.getCurrentBaseSite()).willReturn(baseSiteModel);
        populator.setConfigurationService(configurationService);
        populator.setBaseSiteService(baseSiteService);
        given(baseStore.getScalapayConfiguration()).willReturn(scalaPayConfigurationModel);
        given(baseStoreService.getCurrentBaseStore()).willReturn(baseStore);
        CartModel cart = new CartModel();
        cart.setCode("test-code");
        cart.setCurrency(createEurCurrency());
        cart.setPaymentAddress(createAddress());
        cart.setDeliveryAddress(createAddress());
        cart.setEntries(createEntries());
        cart.setDiscounts(createDiscounts());

        OrderTokenRequest request = new OrderTokenRequest();
        populator.populate(cart, request);

        assertNotNull(request);
        assertEquals(FIRST_NAME, request.getConsumer().getGivenNames());
        compareWithExpectedJson(request);

    }

    private void compareWithExpectedJson(OrderTokenRequest request) throws IOException {
        File file = new File(System.getProperty("user.dir") + "/resources/expectedResult_NEW.json");
        ObjectMapper mapper = new ObjectMapper();
        String json = FileUtils.readFileToString(file, "UTF8");
        OrderTokenRequest expected = mapper.readValue(json, OrderTokenRequest.class);
        assertEquals(expected, request);
    }

    private CurrencyModel createEurCurrency() {
        CurrencyModel eur = new CurrencyModel();
        eur.setIsocode("EUR");
        return eur;
    }

    private List<DiscountModel> createDiscounts() {
        ArrayList<DiscountModel> discountModels = new ArrayList<>();
        discountModels.add(createDiscount());
        return discountModels;
    }

    private DiscountModel createDiscount() {
        DiscountModel discountModel = new StubbedDiscountModel();
        discountModel.setCode("test-promo");
        discountModel.setCurrency(createEurCurrency());
        discountModel.setValue(10d);
        return discountModel;
    }

    private List<AbstractOrderEntryModel> createEntries() {
        ArrayList<AbstractOrderEntryModel> abstractOrderEntryModels = new ArrayList<>();
        OrderEntryModel entry = new OrderEntryModel();
        ProductModel product = new StubbedProductModel();
        product.setName("test-product");
        product.setCode("code-123");
        entry.setProduct(product);
        entry.setQuantity(1L);
        entry.setTotalPrice(20d);
        abstractOrderEntryModels.add(entry);
        return abstractOrderEntryModels;
    }

    private AddressModel createAddress() {
        AddressModel addressModel = new AddressModel();
        addressModel.setFirstname(FIRST_NAME);
        addressModel.setLastname(LAST_NAME);
        CountryModel country = new CountryModel();
        country.setIsocode("IT");
        addressModel.setCountry(country);
        return addressModel;

    }

    static class StubbedProductModel extends ProductModel {
        static ItemModelContext createContext(final Locale loc) {
            final ItemContextBuilder builder = ItemContextBuilder.createDefaultBuilder(ProductModel.class);
            builder.setLocaleProvider(new StubLocaleProvider(loc));
            return builder.build();
        }

        StubbedProductModel() {
            super(createContext(Locale.ITALIAN));
        }
    }

    static class StubbedDiscountModel extends DiscountModel {
        static ItemModelContext createContext(final Locale loc) {
            final ItemContextBuilder builder = ItemContextBuilder.createDefaultBuilder(DiscountModel.class);
            builder.setLocaleProvider(new StubLocaleProvider(loc));
            return builder.build();
        }

        StubbedDiscountModel() {
            super(createContext(Locale.ITALIAN));
        }
    }

}
