package com.scalapay.facade.impl;

import com.scalapay.data.Amount;
import com.scalapay.model.ScalaPayConfigurationModel;
import com.scalapay.service.ScalapayService;
import de.hybris.bootstrap.annotations.UnitTest;
import de.hybris.platform.core.model.c2l.CurrencyModel;
import de.hybris.platform.core.model.order.CartModel;
import de.hybris.platform.order.CartService;
import de.hybris.platform.store.BaseStoreModel;
import de.hybris.platform.store.services.BaseStoreService;
import org.junit.Before;
import org.junit.Test;
import org.junit.runner.RunWith;
import org.mockito.InjectMocks;
import org.mockito.Mock;
import org.mockito.runners.MockitoJUnitRunner;

import static org.junit.Assert.assertFalse;
import static org.junit.Assert.assertTrue;
import static org.mockito.BDDMockito.given;

@UnitTest
@RunWith(MockitoJUnitRunner.class)
public class DefaultScalapayFacadeTest {

    public static final String MIN_AMOUNT = "5";
    public static final String MAX_AMOUNT = "600";
    public static final String CURRENCY_CODE_EUR = "EUR";

    @InjectMocks
    private final DefaultScalapayFacade scalapayFacade = new DefaultScalapayFacade();

    @Mock
    private ScalapayService scalapayService;

    @Mock
    private BaseStoreService baseStoreService;

    @Mock
    private CartService cartService;

    @Mock
    private ScalaPayConfigurationModel scalaPayConfigurationMock;

    @Mock
    private BaseStoreModel baseStoreMock;

    @Mock
    private ScalaPayConfigurationModel configurationMock;

    @Mock
    private org.apache.commons.configuration.Configuration configuration;


    @Before
    public void setUp() {

        configurationMock.setMinAmount(Integer.valueOf(MIN_AMOUNT));
        configurationMock.setMaxAmount(Integer.valueOf(MAX_AMOUNT));
        //given(scalapayService.getScalaPayConfiguration()).willReturn(configurationMock);
        given(baseStoreMock.getScalapayConfiguration()).willReturn(scalaPayConfigurationMock);
        given(baseStoreService.getCurrentBaseStore()).willReturn(baseStoreMock);
        scalapayFacade.setScalapayService(scalapayService);
        // scalapayFacade.setBaseStoreService(baseStoreService);
        //given(configuration.getInt(DefaultScalapayFacade.CACHE_EXPIRY_HOURS)).willReturn(24);
    }

    private Amount createAmount(String amount) {
        Amount minAmount = new Amount();
        minAmount.setCurrency(CURRENCY_CODE_EUR);
        minAmount.setAmount(amount);
        return minAmount;
    }

    @Test
    public void shouldNotBeAvailableOutsideMinAndMax() {
        given(cartService.getSessionCart()).willReturn(generateCartWithValue(Double.parseDouble(MIN_AMOUNT) - 1));
        scalapayFacade.setCartService(cartService);
        assertFalse(scalapayFacade.validForSessionCart());

        given(cartService.getSessionCart()).willReturn(generateCartWithValue(Double.parseDouble(MAX_AMOUNT) + 1));
        scalapayFacade.setCartService(cartService);
        assertFalse(scalapayFacade.validForSessionCart());
    }

    @Test
    public void shouldNotBeAvailableForUnsupportedCurrency() {
        given(cartService.getSessionCart()).willReturn(generateCartWithCurrency("USD"));
        scalapayFacade.setCartService(cartService);
        assertFalse(scalapayFacade.validForSessionCart());
    }

    //@Test todo fix test
    public void shouldBeAvailableWithinMinAndMax() {
        given(cartService.getSessionCart()).willReturn(generateCartWithValue(Double.parseDouble(MIN_AMOUNT) + 1));

        scalapayFacade.setCartService(cartService);
        assertTrue(scalapayFacade.validForSessionCart());

        given(cartService.getSessionCart()).willReturn(generateCartWithValue(Double.parseDouble(MAX_AMOUNT) - 1));
        scalapayFacade.setCartService(cartService);
        assertTrue(scalapayFacade.validForSessionCart());
    }

    private CartModel generateCartWithValue(double totalPrice) {
        CartModel cart = new CartModel();
        cart.setTotalPrice(totalPrice);
        CurrencyModel eur = new CurrencyModel();
        eur.setIsocode("EUR");
        cart.setCurrency(eur);
        return cart;
    }

    private CartModel generateCartWithCurrency(String currency) {
        CartModel cart = new CartModel();
        cart.setTotalPrice(Double.parseDouble(MAX_AMOUNT) + 1);
        CurrencyModel eur = new CurrencyModel();
        eur.setIsocode(currency);
        cart.setCurrency(eur);
        return cart;
    }


}
